@@ -683,7 +683,7 @@ animation_animator_new_frame (ClutterTimeline  *timeline,
       /* only change values if we active (delayed start) */
       if (sub_progress >= 0.0 && sub_progress <= 1.0)
         {
-          GValue tmp_value = G_VALUE_INIT;
+          GValue tmp_value = { 0, };
           GType int_type;
 
           g_value_init (&tmp_value, G_VALUE_TYPE (&start_key->value));
@@ -801,7 +801,7 @@ animation_animator_started (ClutterTimeline *timeline,
 
         if (property_iter->ease_in)
           {
-            GValue tmp_value = G_VALUE_INIT;
+            GValue tmp_value = { 0, };
             GType int_type;
 
             int_type = clutter_interval_get_value_type (property_iter->interval);
@@ -1202,7 +1202,7 @@ clutter_animator_set (ClutterAnimator *animator,
     {
       GParamSpec *pspec;
       GObjectClass *klass;
-      GValue value = G_VALUE_INIT;
+      GValue value = { 0, };
       gchar *error = NULL;
 
       g_return_if_fail (object);
@@ -1458,7 +1458,7 @@ again:
   g_hash_table_remove_all (priv->properties);
 
   /* if the animator is already running reinitialize internal iterators */
-  if (priv->timeline != NULL && clutter_timeline_is_playing (priv->timeline))
+  if (clutter_timeline_is_playing (priv->timeline))
     animation_animator_started (priv->timeline, animator);
 }
 
@@ -1513,9 +1513,9 @@ resolve_interpolation (JsonNode *node)
       gboolean res;
       gint enum_value;
 
-      res = _clutter_script_enum_from_string (CLUTTER_TYPE_INTERPOLATION,
-                                              str,
-                                              &enum_value);
+      res = clutter_script_enum_from_string (CLUTTER_TYPE_INTERPOLATION,
+                                             str,
+                                             &enum_value);
       if (res)
         return enum_value;
     }
@@ -1620,7 +1620,7 @@ parse_animator_property (JsonArray *array,
       gboolean res;
 
       progress = json_array_get_double_element (key, 0);
-      mode = _clutter_script_resolve_animation_mode (json_array_get_element (key, 1));
+      mode = clutter_script_resolve_animation_mode (json_array_get_element (key, 1));
 
       animator_key = clutter_animator_key_new (clos->animator,
                                                gobject,
@@ -1628,11 +1628,11 @@ parse_animator_property (JsonArray *array,
                                                progress,
                                                mode);
 
-      res = _clutter_script_parse_node (clos->script,
-                                        &(animator_key->value),
-                                        pname,
-                                        json_array_get_element (key, 2),
-                                        pspec);
+      res = clutter_script_parse_node (clos->script,
+                                       &(animator_key->value),
+                                       pname,
+                                       json_array_get_element (key, 2),
+                                       pspec);
       if (!res)
         {
           g_warning ("Unable to parse the key value for the "
