@@ -28,15 +28,15 @@
  * @short_description: An action for drop targets
  *
  * #ClutterDropAction is a #ClutterAction that allows a #ClutterActor
- * implementation to control what happens when an actor dragged using
- * a #ClutterDragAction crosses the target area or when a dragged actor
- * is released (or "dropped") on the target area.
+ * implementation to control what happens when a dragged actor crosses
+ * the target area or when a dragged actor is released (or "dropped")
+ * on the target area.
  *
  * A trivial use of #ClutterDropAction consists in connecting to the
  * #ClutterDropAction::drop signal and handling the drop from there,
  * for instance:
  *
- * |[<!-- language="C" -->
+ * |[
  *   ClutterAction *action = clutter_drop_action ();
  *
  *   g_signal_connect (action, "drop", G_CALLBACK (on_drop), NULL);
@@ -49,12 +49,6 @@
  * cause the #ClutterDropAction::drop signal to be skipped when the input
  * device button is released.
  *
- * It's important to note that #ClutterDropAction will only work with
- * actors dragged using #ClutterDragAction.
- *
- * See [drop-action.c](https://git.gnome.org/browse/clutter/tree/examples/drop-action.c?h=clutter-1.18)
- * for an example of how to use #ClutterDropAction.
- *
  * #ClutterDropAction is available since Clutter 1.8
  */
 
@@ -95,14 +89,13 @@ enum
   OVER_IN,
   OVER_OUT,
   DROP,
-  DROP_CANCEL,
 
   LAST_SIGNAL
 };
 
 static guint drop_signals[LAST_SIGNAL] = { 0, };
 
-G_DEFINE_TYPE_WITH_PRIVATE (ClutterDropAction, clutter_drop_action, CLUTTER_TYPE_ACTION)
+G_DEFINE_TYPE (ClutterDropAction, clutter_drop_action, CLUTTER_TYPE_ACTION)
 
 static void
 drop_target_free (gpointer _data)
@@ -121,42 +114,24 @@ on_stage_capture (ClutterStage *stage,
 {
   DropTarget *data = user_data;
   gfloat event_x, event_y;
+  ClutterInputDevice *device;
   ClutterActor *actor, *drag_actor;
   ClutterDropAction *drop_action;
-  ClutterInputDevice *device;
   gboolean was_reactive;
 
-  switch (clutter_event_type (event))
-    {
-    case CLUTTER_MOTION:
-    case CLUTTER_BUTTON_RELEASE:
-      if (clutter_event_type (event) == CLUTTER_MOTION &&
-          !(clutter_event_get_state (event) & CLUTTER_BUTTON1_MASK))
-        return CLUTTER_EVENT_PROPAGATE;
+  if (!(clutter_event_type (event) == CLUTTER_MOTION ||
+        clutter_event_type (event) == CLUTTER_BUTTON_RELEASE))
+    return FALSE;
 
-      if (clutter_event_type (event) == CLUTTER_BUTTON_RELEASE &&
-          clutter_event_get_button (event) != CLUTTER_BUTTON_PRIMARY)
-        return CLUTTER_EVENT_PROPAGATE;
-
-      device = clutter_event_get_device (event);
-      drag_actor = _clutter_stage_get_pointer_drag_actor (stage, device);
-      if (drag_actor == NULL)
-        return CLUTTER_EVENT_PROPAGATE;
-      break;
-
-    case CLUTTER_TOUCH_UPDATE:
-    case CLUTTER_TOUCH_END:
-      drag_actor = _clutter_stage_get_touch_drag_actor (stage,
-                                                        clutter_event_get_event_sequence (event));
-      if (drag_actor == NULL)
-        return CLUTTER_EVENT_PROPAGATE;
-      break;
-
-    default:
-      return CLUTTER_EVENT_PROPAGATE;
-    }
+  if (!(clutter_event_get_state (event) & CLUTTER_BUTTON1_MASK))
+    return FALSE;
 
   clutter_event_get_coords (event, &event_x, &event_y);
+  device = clutter_event_get_device (event);
+
+  drag_actor = _clutter_stage_get_drag_actor (stage, device);
+  if (drag_actor == NULL)
+    return FALSE;
 
   /* get the actor under the cursor, excluding the dragged actor; we
    * use reactivity because it won't cause any scene invalidation
@@ -222,8 +197,7 @@ on_stage_capture (ClutterStage *stage,
     }
 
 out:
-  if (clutter_event_type (event) == CLUTTER_BUTTON_RELEASE ||
-      clutter_event_type (event) == CLUTTER_TOUCH_END)
+  if (clutter_event_type (event) == CLUTTER_BUTTON_RELEASE)
     {
       if (data->last_action != NULL)
         {
@@ -241,13 +215,6 @@ out:
                              clutter_actor_meta_get_actor (meta),
                              event_x, event_y);
             }
-	  else
-            {
-              g_signal_emit (data->last_action, drop_signals[DROP_CANCEL], 0,
-                             clutter_actor_meta_get_actor (meta),
-                             event_x, event_y);
-            }
-
         }
 
       data->last_action = NULL;
@@ -256,7 +223,7 @@ out:
   if (drag_actor != NULL)
     clutter_actor_set_reactive (drag_actor, was_reactive);
 
-  return CLUTTER_EVENT_PROPAGATE;
+  return FALSE;
 }
 
 static void
@@ -289,11 +256,9 @@ static void
 drop_action_unregister (ClutterDropAction *self)
 {
   ClutterDropActionPrivate *priv = self->priv;
-  DropTarget *data = NULL;
-
-  if (priv->stage != NULL)
-    data = g_object_get_data (G_OBJECT (priv->stage), "__clutter_drop_targets");
+  DropTarget *data;
 
+  data = g_object_get_data (G_OBJECT (priv->stage), "__clutter_drop_targets");
   if (data == NULL)
     return;
 
@@ -307,7 +272,7 @@ on_actor_mapped (ClutterActor      *actor,
                  GParamSpec        *pspec,
                  ClutterDropAction *self)
 {
-  if (clutter_actor_is_mapped (actor))
+  if (CLUTTER_ACTOR_IS_MAPPED (actor))
     {
       if (self->priv->stage == NULL)
         self->priv->stage = clutter_actor_get_stage (actor);
@@ -380,6 +345,8 @@ clutter_drop_action_class_init (ClutterDropActionClass *klass)
 {
   ClutterActorMetaClass *meta_class = CLUTTER_ACTOR_META_CLASS (klass);
 
+  g_type_class_add_private (klass, sizeof (ClutterDropActionPrivate));
+
   meta_class->set_actor = clutter_drop_action_set_actor;
 
   klass->can_drop = clutter_drop_action_real_can_drop;
@@ -388,8 +355,6 @@ clutter_drop_action_class_init (ClutterDropActionClass *klass)
    * ClutterDropAction::can-drop:
    * @action: the #ClutterDropAction that emitted the signal
    * @actor: the #ClutterActor attached to the @action
-   * @event_x: the X coordinate (in stage space) of the drop event
-   * @event_y: the Y coordinate (in stage space) of the drop event
    *
    * The ::can-drop signal is emitted when the dragged actor is dropped
    * on @actor. The return value of the ::can-drop signal will determine
@@ -459,8 +424,6 @@ clutter_drop_action_class_init (ClutterDropActionClass *klass)
    * ClutterDropAction::drop:
    * @action: the #ClutterDropAction that emitted the signal
    * @actor: the #ClutterActor attached to the @action
-   * @event_x: the X coordinate (in stage space) of the drop event
-   * @event_y: the Y coordinate (in stage space) of the drop event
    *
    * The ::drop signal is emitted when the dragged actor is dropped
    * on @actor. This signal is only emitted if at least an handler of
@@ -479,40 +442,13 @@ clutter_drop_action_class_init (ClutterDropActionClass *klass)
                   CLUTTER_TYPE_ACTOR,
                   G_TYPE_FLOAT,
                   G_TYPE_FLOAT);
-
-
-  /**
-   * ClutterDropAction::drop-cancel:
-   * @action: the #ClutterDropAction that emitted the signal
-   * @actor: the #ClutterActor attached to the @action
-   * @event_x: the X coordinate (in stage space) of the drop event
-   * @event_y: the Y coordinate (in stage space) of the drop event
-   *
-   * The ::drop-cancel signal is emitted when the drop is refused
-   * by an emission of the #ClutterDropAction::can-drop signal.
-   *
-   * After the ::drop-cancel signal is fired the active drag is
-   * terminated.
-   *
-   * Since: 1.12
-   */
-  drop_signals[DROP_CANCEL] =
-    g_signal_new (I_("drop-cancel"),
-                  G_TYPE_FROM_CLASS (klass),
-                  G_SIGNAL_RUN_LAST,
-                  G_STRUCT_OFFSET (ClutterDropActionClass, drop),
-                  NULL, NULL,
-                  _clutter_marshal_VOID__OBJECT_FLOAT_FLOAT,
-                  G_TYPE_NONE, 3,
-                  CLUTTER_TYPE_ACTOR,
-                  G_TYPE_FLOAT,
-                  G_TYPE_FLOAT);
 }
 
 static void
 clutter_drop_action_init (ClutterDropAction *self)
 {
-  self->priv = clutter_drop_action_get_instance_private (self);
+  self->priv = G_TYPE_INSTANCE_GET_PRIVATE (self, CLUTTER_TYPE_DROP_ACTION,
+                                            ClutterDropActionPrivate);
 }
 
 /**
