@@ -42,11 +42,13 @@
  *   <title>Using a Layout Manager inside an Actor</title>
  *   <para>In order to use a #ClutterLayoutManager inside a #ClutterActor
  *   sub-class you should invoke clutter_layout_manager_get_preferred_width()
- *   inside the #ClutterActorClass.get_preferred_width() virtual function and
- *   clutter_layout_manager_get_preferred_height() inside the
- *   #ClutterActorClass.get_preferred_height() virtual functions implementation.
- *   You should also call clutter_layout_manager_allocate() inside the
- *   implementation of the #ClutterActorClass.allocate() virtual function.</para>
+ *   inside the <structname>ClutterActor</structname>::get_preferred_width()
+ *   virtual function and clutter_layout_manager_get_preferred_height()
+ *   inside the <structname>ClutterActor</structname>::get_preferred_height()
+ *   virtual function implementations. You should also call
+ *   clutter_layout_manager_allocate() inside the implementation of the
+ *   <structname>ClutterActor</structname>::allocate() virtual
+ *   function.</para>
  *   <para>In order to receive notifications for changes in the layout
  *   manager policies you should also connect to the
  *   #ClutterLayoutManager::layout-changed signal and queue a relayout
@@ -73,10 +75,38 @@
  *   <function>set_container()</function> virtual function. The layout manager
  *   should not hold a real reference (i.e. call g_object_ref()) on the
  *   container actor, to avoid reference cycles.</para>
- *   <para>If a layout manager has properties affecting the layout
+ *   <para>If the layout manager has properties affecting the layout
  *   policies then it should emit the #ClutterLayoutManager::layout-changed
  *   signal on itself by using the clutter_layout_manager_layout_changed()
  *   function whenever one of these properties changes.</para>
+ *   <para>If the layout manager has layout properties, that is properties that
+ *   should exist only as the result of the presence of a specific (layout
+ *   manager, container actor, child actor) combination, and it wishes to store
+ *   those properties inside a #ClutterLayoutMeta then it should override the
+ *   <structname>ClutterLayoutManager</structname>::get_child_meta_type()
+ *   virtual function to return the #GType of the #ClutterLayoutMeta sub-class
+ *   used to store the layout properties; optionally, the #ClutterLayoutManager
+ *   sub-class might also override the
+ *   <structname>ClutterLayoutManager</structname>::create_child_meta() virtual
+ *   function to control how the #ClutterLayoutMeta instance is created,
+ *   otherwise the default implementation will be equivalent to:</para>
+ *   <informalexample><programlisting>
+ *  ClutterLayoutManagerClass *klass;
+ *  GType meta_type;
+ *
+ *  klass = CLUTTER_LAYOUT_MANAGER_GET_CLASS (manager);
+ *  meta_type = klass->get_child_meta_type (manager);
+ *
+ *  return g_object_new (meta_type,
+ *                       "manager", manager,
+ *                       "container", container,
+ *                       "actor", actor,
+ *                       NULL);
+ *   </programlisting></informalexample>
+ *   <para>Where <varname>manager</varname> is the  #ClutterLayoutManager,
+ *   <varname>container</varname> is the #ClutterContainer using the
+ *   #ClutterLayoutManager and <varname>actor</varname> is the #ClutterActor
+ *   child of the #ClutterContainer.</para>
  * </refsect2>
  *
  * <refsect2 id="ClutterLayoutManager-animation">
@@ -91,9 +121,9 @@
  *   <para>It is possible for a #ClutterLayoutManager sub-class to animate its
  *   children layout by using the base class animation support. The
  *   #ClutterLayoutManager animation support consists of three virtual
- *   functions: #ClutterLayoutManagerClass.begin_animation(),
- *   #ClutterLayoutManagerClass.get_animation_progress(), and
- *   #ClutterLayoutManagerClass.end_animation().</para>
+ *   functions: <function>begin_animation()</function>,
+ *   <function>get_animation_progress()</function> and
+ *   <function>end_animation()</function>.</para>
  *   <variablelist>
  *     <varlistentry>
  *       <term><function>begin_animation (duration, easing)</function></term>
@@ -129,20 +159,20 @@
  *   clutter_layout_manager_end_animation() to be called. The default
  *   implementation provided internally by #ClutterLayoutManager does exactly
  *   this, so most sub-classes should either not override any animation-related
- *   virtual function or simply override #ClutterLayoutManagerClass.begin_animation()
- *   and #ClutterLayoutManagerClass.end_animation() to set up ad hoc state, and then
+ *   virtual function or simply override <function>begin_animation()</function>
+ *   and <function>end_animation()</function> to set up ad hoc state, and then
  *   chain up to the parent's implementation.</para>
  *   <example id="example-ClutterLayoutManager-animation">
  *     <title>Animation of a Layout Manager</title>
  *     <para>The code below shows how a #ClutterLayoutManager sub-class should
  *     provide animating the allocation of its children from within the
- *     #ClutterLayoutManagerClass.allocate() virtual function implementation. The
+ *     <function>allocate()</function> virtual function implementation. The
  *     animation is computed between the last stable allocation performed
  *     before the animation started and the desired final allocation.</para>
  *     <para>The <varname>is_animating</varname> variable is stored inside the
  *     #ClutterLayoutManager sub-class and it is updated by overriding the
- *     #ClutterLayoutManagerClass.begin_animation() and the
- *     #ClutterLayoutManagerClass.end_animation() virtual functions and chaining up
+ *     <function>begin_animation()</function> and
+ *     <function>end_animation()</function> virtual functions and chaining up
  *     to the base class implementation.</para>
  *     <para>The last stable allocation is stored within a #ClutterLayoutMeta
  *     sub-class used by the implementation.</para>
@@ -154,12 +184,13 @@
  *                             ClutterAllocationFlags  flags)
  * {
  *   MyLayoutManager *self = MY_LAYOUT_MANAGER (manager);
- *   ClutterActor *child;
+ *   GList *children, *l;
  *
- *   for (child = clutter_actor_get_first_child (CLUTTER_ACTOR (container));
- *        child != NULL;
- *        child = clutter_actor_get_next_sibling (child))
+ *   children = clutter_container_get_children (container);
+ *
+ *   for (l = children; l != NULL; l = l-&gt;next)
  *     {
+ *       ClutterActor *child = l->data;
  *       ClutterLayoutMeta *meta;
  *       MyLayoutMeta *my_meta;
  *
@@ -220,6 +251,8 @@
  *       /&ast; allocate the child &ast;/
  *       clutter_actor_allocate (child, &child_box, flags);
  *     }
+ *
+ *   g_list_free (children);
  * }
  *     </programlisting>
  *   </example>
@@ -246,77 +279,6 @@
  *   <varname>orientation</varname> layout property of a layout manager.</para>
  * </refsect2>
  *
- * <refsect2 id="clutter-layout-properties">
- *   <title>Layout Properties</title>
- *   <para>If a layout manager has layout properties, that is properties that
- *   should exist only as the result of the presence of a specific (layout
- *   manager, container actor, child actor) combination, and it wishes to store
- *   those properties inside a #ClutterLayoutMeta, then it should override the
- *   #ClutterLayoutManagerClass.get_child_meta_type() virtual function to return
- *   the #GType of the #ClutterLayoutMeta sub-class used to store the layout
- *   properties; optionally, the #ClutterLayoutManager sub-class might also
- *   override the #ClutterLayoutManagerClass.create_child_meta() virtual function
- *   to control how the #ClutterLayoutMeta instance is created, otherwise the
- *   default implementation will be equivalent to:</para>
- *   <informalexample><programlisting>
- *  ClutterLayoutManagerClass *klass;
- *  GType meta_type;
- *
- *  klass = CLUTTER_LAYOUT_MANAGER_GET_CLASS (manager);
- *  meta_type = klass->get_child_meta_type (manager);
- *
- *  return g_object_new (meta_type,
- *                       "manager", manager,
- *                       "container", container,
- *                       "actor", actor,
- *                       NULL);
- *   </programlisting></informalexample>
- *   <para>Where <varname>manager</varname> is the  #ClutterLayoutManager,
- *   <varname>container</varname> is the #ClutterContainer using the
- *   #ClutterLayoutManager and <varname>actor</varname> is the #ClutterActor
- *   child of the #ClutterContainer.</para>
- * </refsect2>
- *
- * <refsect2 id="clutter-layout-script">
- *   <title>Using ClutterLayoutManager with ClutterScript</title>
- *   <para>#ClutterLayoutManager instance can be created in the same way
- *   as other objects in #ClutterScript; properties can be set using the
- *   common syntax.</para>
- *   <para>Layout properties can be set on children of a container with
- *   a #ClutterLayoutManager using the <emphasis>layout::</emphasis>
- *   modifier on the property name, for instance:</para>
- *   <informalexample><programlisting>
- * {
- *   "type" : "ClutterBox",
- *   "layout-manager" : { "type" : "ClutterTableLayout" },
- *   "children" : [
- *     {
- *       "type" : "ClutterTexture",
- *       "filename" : "image-00.png",
- *
- *       "layout::row" : 0,
- *       "layout::column" : 0,
- *       "layout::x-align" : "left",
- *       "layout::y-align" : "center",
- *       "layout::x-expand" : true,
- *       "layout::y-expand" : true
- *     },
- *     {
- *       "type" : "ClutterTexture",
- *       "filename" : "image-01.png",
- *
- *       "layout::row" : 0,
- *       "layout::column" : 1,
- *       "layout::x-align" : "right",
- *       "layout::y-align" : "center",
- *       "layout::x-expand" : true,
- *       "layout::y-expand" : true
- *     }
- *   ]
- * }
- *   </programlisting></informalexample>
- * </refsect2>
- *
  * #ClutterLayoutManager is available since Clutter 1.2
  */
 
@@ -327,10 +289,7 @@
 #include <glib-object.h>
 #include <gobject/gvaluecollector.h>
 
-#define CLUTTER_DISABLE_DEPRECATION_WARNINGS
-#include "deprecated/clutter-container.h"
-#include "deprecated/clutter-alpha.h"
-
+#include "clutter-alpha.h"
 #include "clutter-debug.h"
 #include "clutter-layout-manager.h"
 #include "clutter-layout-meta.h"
@@ -354,7 +313,7 @@ enum
 
 G_DEFINE_ABSTRACT_TYPE (ClutterLayoutManager,
                         clutter_layout_manager,
-                        G_TYPE_INITIALLY_UNOWNED)
+                        G_TYPE_INITIALLY_UNOWNED);
 
 static GQuark quark_layout_meta  = 0;
 static GQuark quark_layout_alpha = 0;
@@ -494,7 +453,6 @@ layout_manager_real_get_child_meta_type (ClutterLayoutManager *manager)
   return G_TYPE_INVALID;
 }
 
-/* XXX:2.0 - Remove */
 static ClutterAlpha *
 layout_manager_real_begin_animation (ClutterLayoutManager *manager,
                                      guint                 duration,
@@ -538,7 +496,6 @@ layout_manager_real_begin_animation (ClutterLayoutManager *manager,
   return alpha;
 }
 
-/* XXX:2.0 - Remove */
 static gdouble
 layout_manager_real_get_animation_progress (ClutterLayoutManager *manager)
 {
@@ -551,7 +508,6 @@ layout_manager_real_get_animation_progress (ClutterLayoutManager *manager)
   return clutter_alpha_get_alpha (alpha);
 }
 
-/* XXX:2.0 - Remove */
 static void
 layout_manager_real_end_animation (ClutterLayoutManager *manager)
 {
@@ -585,8 +541,6 @@ clutter_layout_manager_class_init (ClutterLayoutManagerClass *klass)
 {
   quark_layout_meta =
     g_quark_from_static_string ("clutter-layout-manager-child-meta");
-
-  /* XXX:2.0 - Remove */
   quark_layout_alpha =
     g_quark_from_static_string ("clutter-layout-manager-alpha");
 
@@ -595,8 +549,6 @@ clutter_layout_manager_class_init (ClutterLayoutManagerClass *klass)
   klass->allocate = layout_manager_real_allocate;
   klass->create_child_meta = layout_manager_real_create_child_meta;
   klass->get_child_meta_type = layout_manager_real_get_child_meta_type;
-
-  /* XXX:2.0 - Remove */
   klass->begin_animation = layout_manager_real_begin_animation;
   klass->get_animation_progress = layout_manager_real_get_animation_progress;
   klass->end_animation = layout_manager_real_end_animation;
@@ -987,7 +939,7 @@ clutter_layout_manager_child_set (ClutterLayoutManager *manager,
   pname = first_property;
   while (pname)
     {
-      GValue value = G_VALUE_INIT;
+      GValue value = { 0, };
       GParamSpec *pspec;
       gchar *error;
       gboolean res;
@@ -1127,7 +1079,7 @@ clutter_layout_manager_child_get (ClutterLayoutManager *manager,
   pname = first_property;
   while (pname)
     {
-      GValue value = G_VALUE_INIT;
+      GValue value = { 0, };
       GParamSpec *pspec;
       gchar *error;
       gboolean res;
@@ -1301,3 +1253,81 @@ clutter_layout_manager_list_child_properties (ClutterLayoutManager *manager,
 
   return pspecs;
 }
+
+/**
+ * clutter_layout_manager_begin_animation:
+ * @manager: a #ClutterLayoutManager
+ * @duration: the duration of the animation, in milliseconds
+ * @mode: the easing mode of the animation
+ *
+ * Begins an animation of @duration milliseconds, using the provided
+ * easing @mode
+ *
+ * The easing mode can be specified either as a #ClutterAnimationMode
+ * or as a logical id returned by clutter_alpha_register_func()
+ *
+ * The result of this function depends on the @manager implementation
+ *
+ * Return value: (transfer none): The #ClutterAlpha created by the
+ *   layout manager; the returned instance is owned by the layout
+ *   manager and should not be unreferenced
+ *
+ * Since: 1.2
+ */
+ClutterAlpha *
+clutter_layout_manager_begin_animation (ClutterLayoutManager *manager,
+                                        guint                 duration,
+                                        gulong                mode)
+{
+  ClutterLayoutManagerClass *klass;
+
+  g_return_val_if_fail (CLUTTER_IS_LAYOUT_MANAGER (manager), NULL);
+
+  klass = CLUTTER_LAYOUT_MANAGER_GET_CLASS (manager);
+
+  return klass->begin_animation (manager, duration, mode);
+}
+
+/**
+ * clutter_layout_manager_end_animation:
+ * @manager: a #ClutterLayoutManager
+ *
+ * Ends an animation started by clutter_layout_manager_begin_animation()
+ *
+ * The result of this call depends on the @manager implementation
+ *
+ * Since: 1.2
+ */
+void
+clutter_layout_manager_end_animation (ClutterLayoutManager *manager)
+{
+  g_return_if_fail (CLUTTER_IS_LAYOUT_MANAGER (manager));
+
+  CLUTTER_LAYOUT_MANAGER_GET_CLASS (manager)->end_animation (manager);
+}
+
+/**
+ * clutter_layout_manager_get_animation_progress:
+ * @manager: a #ClutterLayoutManager
+ *
+ * Retrieves the progress of the animation, if one has been started by
+ * clutter_layout_manager_begin_animation()
+ *
+ * The returned value has the same semantics of the #ClutterAlpha:alpha
+ * value
+ *
+ * Return value: the progress of the animation
+ *
+ * Since: 1.2
+ */
+gdouble
+clutter_layout_manager_get_animation_progress (ClutterLayoutManager *manager)
+{
+  ClutterLayoutManagerClass *klass;
+
+  g_return_val_if_fail (CLUTTER_IS_LAYOUT_MANAGER (manager), 1.0);
+
+  klass = CLUTTER_LAYOUT_MANAGER_GET_CLASS (manager);
+
+  return klass->get_animation_progress (manager);
+}
