@@ -32,9 +32,7 @@
 #include <sys/types.h>
 #include <sys/uio.h>
 #include <unistd.h>
-
-#include "clutter-debug.h"
-#include "clutter-private.h"
+#include <clutter/clutter-debug.h>
 
 /* 
  * This file implementations integration between the GLib main loop and
@@ -526,8 +524,8 @@ select_thread_start_poll (GPollFD *ufds,
 	have_new_pollfds = TRUE;
       else
 	{
-	  if (!((nfds == 1 && poll_fd_index < 0) ||
-		(nfds == 2 && poll_fd_index >= 0)))
+	  if (!((nfds == 1 && poll_fd_index < 0 && g_thread_supported ()) ||
+		(nfds == 2 && poll_fd_index >= 0 && g_thread_supported ())))
 	    select_thread_set_state (POLLING_RESTART);
 	}
     }
@@ -645,13 +643,13 @@ clutter_event_prepare (GSource *source,
 {
   gboolean retval;
 
-  _clutter_threads_acquire_lock ();
+  clutter_threads_enter ();
   
   *timeout = -1;
 
   retval = (clutter_events_pending () || _clutter_osx_event_loop_check_pending ());
 
-  _clutter_threads_release_lock ();
+  clutter_threads_leave ();
 
   return retval;
 }
@@ -661,7 +659,7 @@ clutter_event_check (GSource *source)
 {
   gboolean retval;
 
-  _clutter_threads_acquire_lock ();
+  clutter_threads_enter ();
 
   /* XXX: This check isn't right it won't handle a recursive GLib main
    * loop run within an outer CFRunLoop run. Such loops will pile up
@@ -680,7 +678,7 @@ clutter_event_check (GSource *source)
   
   retval = (clutter_events_pending () || _clutter_osx_event_loop_check_pending ());
 
-  _clutter_threads_release_lock ();
+  clutter_threads_leave ();
 
   return retval;
 }
@@ -693,19 +691,16 @@ clutter_event_dispatch (GSource     *source,
   NSEvent *nsevent;
   ClutterEvent *event;
 
-  _clutter_threads_acquire_lock ();
+  clutter_threads_enter ();
 
   nsevent = _clutter_osx_event_loop_get_pending ();
-  if (nsevent)
-    {
-      _clutter_threads_release_lock ();
-
-      [NSApp sendEvent:nsevent];
-
-      _clutter_threads_acquire_lock ();
-
-      _clutter_osx_event_loop_release_event (nsevent);
-    }
+  if (nsevent) {
+    clutter_threads_leave ();
+    [NSApp sendEvent:nsevent];
+    clutter_threads_enter ();
+    
+    _clutter_osx_event_loop_release_event (nsevent);
+  }
   
   event = clutter_event_get ();
 
@@ -716,7 +711,7 @@ clutter_event_dispatch (GSource     *source,
       clutter_event_free (event);
     }
 
-  _clutter_threads_release_lock ();
+  clutter_threads_leave ();
 
   return TRUE;
 }
